import logging
import os

from nics.main.constants import __version__


logger = logging.getLogger(__name__)


def get_text(
    author, custom_license, color_hue, lowercase_the_url, show_credit, char_map,
    git_name, git_email, gh_username, gh_repo_name, load, dock, container
):
    return f"""
#-- Welcome to NICS settings!
#-- Everything starts with "#--" is a comment.
#-- Documentation: https://nvfp.github.io/now-i-can-sleep


author: '{author}'
custom_license: {custom_license}
color_hue: {color_hue}
lowercase_the_url: {lowercase_the_url}
show_credit: {show_credit}
char_map: {char_map}


#-- The variables below are for NICS internal use only and shouldn't be modified.
_git_name: '{git_name}'
_git_email: '{git_email}'
_gh_username: '{gh_username}'
_gh_repo_name: '{gh_repo_name}'
_load: '{load}'
_dock: '{dock}'
_container: '{container}'
_nics_version: '{__version__}'
"""


def settings_writer(
    load_path,
    author, custom_license, color_hue, lowercase_the_url, show_credit, char_map,
    git_name, git_email, gh_username, gh_repo_name, load, dock, container
):
    logger.debug('Writing settings.txt file.')

    file_path = os.path.join(load_path, container, 'settings.txt')
    text = get_text(
        author, custom_license, color_hue, lowercase_the_url, show_credit, char_map,
        git_name, git_email, gh_username, gh_repo_name, load, dock, container
    )
    with open(file_path, 'w') as f:
        f.write(text)

    logger.debug(f'Done, created at {repr(file_path)}.')


import os

from nics.main.utils.ensure_a_git_repo import ensure_a_git_repo
from nics.main.cmd_init.ensure_nics_env_can_be_installed import ensure_nics_env_can_be_installed
from nics.main.cmd_init.get_user_details import get_user_details
from nics.main.cmd_init.loading import loading
from nics.main.cmd_init.print_outro import print_outro


def run():

    load_path = os.getcwd()

    ## Check I
    ensure_a_git_repo(load_path)

    ## Get user details
    usr = get_user_details()

    ## Check II
    ensure_nics_env_can_be_installed(load_path, usr.container)

    ## Loading to the Container
    loading(
        load_path,
        usr.load, usr.dock, usr.container,
        usr.author, usr.git_name, usr.git_email,
        usr.gh_username, usr.gh_repo_name
    )

    ## Outro
    print_outro(usr.load, usr.dock, usr.gh_username, usr.gh_repo_name)




def print_outro(load, dock, gh_username, gh_repo_name):
    text = f"""
Almost done, now you need to do these final steps:
1. Create {dock} branch
   - git add .
   - git commit -m "NICS init"
   - git checkout --orphan {dock}
   - git rm -rf .
   - git commit --allow-empty -m init
   - git push origin {dock}
2. Activate the GitHub Pages
   - Visit https://github.com/{gh_username}/{gh_repo_name}/settings/pages
   - Under 'Build and deployment' section,
     - For 'Source', select 'Deploy from a branch'
     - For 'Branch', select '{dock}' branch
     - Click the 'Save' button
3. Back to {load} branch
   - git checkout {load}
   - git push

That's it! The documentation will be at https://{gh_username}.github.io/{gh_repo_name} ."""
    print(text)


import argparse
import logging

from nics.main.constants import __version__
from nics.main.cmd_init import run as run_init
from nics.main.cmd_upgrade import run as run_upgrade


logging.basicConfig(format='[%(asctime)s] %(levelname)s: %(message)s', datefmt='%H:%M:%S')
logger = logging.getLogger(__name__)


def main():

    parser = argparse.ArgumentParser(
        usage=(
            '\n'
            '├─ Run `nics init`   : Set up NICS environment\n'
            '└─ Run `nics upgrade`: Reconfigure NICS environment'
        ),
        formatter_class=argparse.RawTextHelpFormatter  # to use line breaks (\n) in the help message
    )
    parser.add_argument('-v', '--version', action='version', version=f'nics-{__version__}', help='show software version')
    parser.add_argument('-L', dest='log_level', choices=['debug', 'info', 'warning', 'error'], default='info', help='Set the log level (default: %(default)s)')
    parser.add_argument('cmd', choices=['init', 'upgrade'], help=argparse.SUPPRESS)  # `help=argparse.SUPPRESS` to hide the default help message

    args = parser.parse_args()
    logger.setLevel(getattr(logging, args.log_level.upper()))

    logger.debug(f'Running {repr(args.cmd)} command.')
    if args.cmd == 'init':
        run_init()
    elif args.cmd == 'upgrade':
        run_upgrade()


import os


def is_nics_initialized(dir_path):
    """
    Shallow check if `dir_path` has been previously initialized with NICS.
    Return True if clues are found, False otherwise.
    """

    clues = []

    ## Inspect /.github/workflows/rebuild-docs.yml
    if os.path.isfile(os.path.join(dir_path, '.github', 'workflows', 'rebuild-docs.yml')):
        clues.append(True)
    else:
        clues.append(False)

    ## Inspect /docs/settings.txt
    if os.path.isfile(os.path.join(dir_path, 'docs', 'settings.txt')):
        clues.append(True)
    else:
        clues.append(False)

    ## Inspect /docs/tree/index.md
    if os.path.isfile(os.path.join(dir_path, 'docs', 'tree', 'index.md')):
        clues.append(True)
    else:
        clues.append(False)

    return all(clues)  # True if all in `clues` are True


import os
import random

from nics.main.cmd_init.loading.workflow_writer import workflow_writer
from nics.main.cmd_init.loading.copy_template import copy_template
from nics.main.utils.page404_maker import create_404_page_file
from nics.main.utils.favicon_maker import create_favicon_svg_file
from nics.main.cmd_init.loading.settings_writer import settings_writer


def loading(load_path, load, dock, container, author, git_name, git_email, gh_username, gh_repo_name):

    ## Write rebuild-docs.yml
    workflow_writer(load_path, load, dock, container, git_name, git_email, gh_repo_name)

    ## Copying template
    copy_template(load_path, container)

    ## Customize 404.md
    create_404_page_file(os.path.join(load_path, container, '404.md'), gh_username, gh_repo_name)

    ## Default user details
    custom_license = None
    color_hue = random.randint(0, 359)
    lowercase_the_url = True
    show_credit = True
    char_map = {}

    ## Customize favicon.svg
    create_favicon_svg_file(os.path.join(load_path, container, 'favicon.svg'), color_hue)

    ## Customize settings.txt
    settings_writer(
        load_path,
        author, custom_license, color_hue, lowercase_the_url, show_credit, char_map,
        git_name, git_email, gh_username, gh_repo_name, load, dock, container
    )


import logging
import os
import sys


logger = logging.getLogger(__name__)


def ensure_nics_env_can_be_installed(load_path, container):

    ## /.github/workflows/rebuild-docs.yml
    pth = os.path.join(load_path, '.github', 'workflows', 'rebuild-docs.yml')
    if os.path.isfile(pth):
        logger.error(f'Cannot initialize NICS. This file already exists: {repr(pth)}.')
        sys.exit(1)

    ## /CONTAINER/
    pth = os.path.join(load_path, container)
    if os.path.isdir(pth):
        logger.error(f'Cannot initialize NICS. This folder already exists: {repr(pth)}.')
        sys.exit(1)


import os
import pkg_resources


try:
    __version__ = pkg_resources.get_distribution('nics').version
except pkg_resources.DistributionNotFound:  # This exception occurred during development (before the software installed via pip)
    __version__ = 'dev'


ROOT_DIR_PTH = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

DIST_DIR_PTH = os.path.join(ROOT_DIR_PTH, 'nics')
MAIN_DIR_PTH = os.path.join(ROOT_DIR_PTH, 'nics', 'main')
NICS_COMPILER_DIR_PTH = os.path.join(ROOT_DIR_PTH, 'nics-compiler')

TEMPLATE_DOCS_DIR_PTH = os.path.join(ROOT_DIR_PTH, 'nics', 'main', 'template')
TEMPLATE_WEB_DIR_PTH  = os.path.join(ROOT_DIR_PTH, 'nics-compiler', 'template')

SETTINGS_KEYS = [
    'author',
    'custom_license',
    'color_hue',
    'lowercase_the_url',
    'show_credit',
    'char_map',
    '_git_name',
    '_git_email',
    '_gh_username',
    '_gh_repo_name',
    '_load',
    '_dock',
    '_container',
    '_nics_version',
]


import logging


logger = logging.getLogger(__name__)


def get_text(gh_username, gh_repo_name):
    return f"""
## Sorry, it seems the page doesn't exists or no longer exists.

Do want to check it on the [homepage](https://{gh_username}.github.io/{gh_repo_name})?
"""


def create_404_page_file(file_path, gh_username, gh_repo_name):
    logger.debug('Writing 404 page file.')

    text = get_text(gh_username, gh_repo_name)
    with open(file_path, 'w') as f:
        f.write(text)
    
    logger.debug(f'404 page created at {repr(file_path)}.')


import logging
import os
import sys

from nics.main.constants import __version__


logger = logging.getLogger(__name__)


def check_rebuild_docs_yml_existence(load_path):

    file_path = os.path.join(load_path, '.github', 'workflows', 'rebuild-docs.yml')

    if not os.path.isfile(file_path):
        logger.error(f'File {repr(file_path)} not found.')
        sys.exit(1)


def check_container_clues(load_path, container):

    ## The homepage file
    file_path = os.path.join(load_path, container, 'tree', 'index.md')
    if not os.path.isfile(file_path):
        logger.error(f'File {repr(file_path)} not found.')
        sys.exit(1)

    ## The settings.txt file
    file_path = os.path.join(load_path, container, 'settings.txt')
    if not os.path.isfile(file_path):
        logger.error(f'File {repr(file_path)} not found.')
        sys.exit(1)


def check_upgrade_status(ver):

    if ver == __version__:
        print('Fantastic! Everything is already up to date.')
        sys.exit(0)


import logging
import os
import sys

from mykit.kit.keycrate import KeyCrate

from nics.main.constants import SETTINGS_KEYS


logger = logging.getLogger(__name__)


def gather_settings(load_path, container):
    
    ## The settings.txt file
    file_path = os.path.join(load_path, container, 'settings.txt')

    old = KeyCrate(file_path, True, True)

    class cfg: ...

    for key in SETTINGS_KEYS:
        try:
            setattr(cfg, key, old[key])
        except AttributeError:
            setattr(cfg, key, input(f'Enter the value of this new option {repr(key)}: '))

    return cfg


import logging
import os
import re
import sys


logger = logging.getLogger(__name__)


def where_is_the_container(load_path):

    ## /.github/workflows/rebuild-docs.yml
    file_path = os.path.join(load_path, '.github', 'workflows', 'rebuild-docs.yml')

    with open(file_path, 'r') as f:
        text = f.read()

    res = re.search(r'container: (?P<container>.*)', text)
    if res is None:
        logger.debug(f'Cannot find container {repr(container)} in {repr(file_path)}.')
        logger.error(
            "Cannot find 'container' in rebuild-docs.yml. "
            "Looks like NICS has not been initialized for this repository. Try 'nics init' instead."
        )
        sys.exit(1)
    else:
        container = res.group('container')
        logger.debug(f'Parsed container {repr(container)} from {repr(file_path)}.')
        return container